package com.contentsquare.flutter

import android.app.Activity
import android.content.Context
import android.view.View
import android.view.ViewGroup
import androidx.annotation.NonNull
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphListener
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphResult
import com.contentsquare.android.api.bridge.flutter.FlutterInterface
import com.contentsquare.android.api.bridge.xpf.ExternalBridgeInterface
import com.contentsquare.android.api.bridge.xpf.ExternalBridgeType
import com.contentsquare.flutter.infrastructure.bridges.FlutterInterfaceBridge
import com.contentsquare.flutter.infrastructure.bridges.XpfInterfaceBridge
import com.contentsquare.flutter.infrastructure.core.method_call_executor.MethodCallExecutor
import com.contentsquare.flutter.infrastructure.core.method_call_executor.agents.MethodCallAgent
import com.contentsquare.flutter.infrastructure.core.method_call_executor.data.ExecutionData
import com.contentsquare.flutter.infrastructure.core.states.ContentsquarePluginState
import com.contentsquare.flutter.infrastructure.core.states.StateObject
import io.flutter.Log
import io.flutter.embedding.android.FlutterImageView
import io.flutter.embedding.android.FlutterView
import io.flutter.embedding.engine.mutatorsstack.FlutterMutatorView
import io.flutter.embedding.engine.plugins.lifecycle.HiddenLifecycleReference

internal class FlutterViewFinder {

    fun find(activity: Activity): View? {
        val view = activity.window.decorView
        return findRecursively(view)
    }

    private fun findRecursively(view: View): View? {
        if (view is FlutterView) {
            return view
        }
        if (view is ViewGroup) {
            for (childIndex in 0 until view.childCount) {
                val viewFound = findRecursively(view.getChildAt(childIndex))
                if (viewFound != null) {
                    return viewFound
                }
            }
        }
        return null
    }
}

/** ContentsquarePlugin */
class ContentsquarePlugin : FlutterPlugin, ActivityAware, MethodCallHandler,
    DefaultLifecycleObserver {
    private val flutterTVPSeparator = "|flutter|"

    private lateinit var channel: MethodChannel
    private var context: Context? = null
    private lateinit var view: View
    private lateinit var activity: Activity
    private lateinit var executor: MethodCallExecutor
    private var featuresState = ContentsquarePluginState
    private var flutterInterfaceBridge = FlutterInterfaceBridge.default()
    private val xpfInterfaceBridge = XpfInterfaceBridge.default()
    private val externalBridgeInterface = object : ExternalBridgeInterface {
        override val bridgeType: ExternalBridgeType = ExternalBridgeType.FLUTTER

        override fun notifyApiErrorsEnabled(enable: Boolean) {
            featuresState.setApiErrorsState(StateObject(enable))
            channel.invokeMethod(
                ChannelConst.Method.REQUEST_ENABLE_API_ERROR,
                mapOf(ChannelConst.Key.ENABLE_API_ERRORS to enable)
            )
        }

        // TODO: change this method to accept the view id as a parameter when the Native SDK will be updated in FLUT-1215
        override fun notifySessionReplayEnabled(enable: Boolean) {
            val containerViewId: Int? = null
            var pixelRatio = 1.0f
            if (enable) {
                pixelRatio = context?.resources?.displayMetrics?.density ?: 1.0f
            }

            featuresState.setContainerViewIdState(StateObject(containerViewId))
            featuresState.setSessionReplayState(StateObject(enable))
            featuresState.setPixelRatioState(StateObject(pixelRatio.toDouble()))

            channel.invokeMethod(
                ChannelConst.Method.REQUEST_ENABLE_SR,
                mapOf(ChannelConst.Key.ENABLE_SR to enable,
                        ChannelConst.Key.CONTAINER_VIEW_ID to containerViewId,
                        ChannelConst.Key.PIXEL_RATIO to pixelRatio)
            )
        }


        override fun notifyCrashReportingEnabled(enable: Boolean) {
            featuresState.setCrashReporterState(StateObject(enable))
            channel.invokeMethod(
                ChannelConst.Method.REQUEST_ENABLE_CRASH_REPORTER,
                mapOf(ChannelConst.Key.ENABLE_CRASH_REPORTER to enable)
            )
        }

        override fun notifyFeatureFlagsEnabled(featureFlags: List<Map<String, Any>>) {
            featuresState.setFeatureFlagsState(StateObject(featureFlags))

            channel.invokeMethod(
                ChannelConst.Method.UPDATE_FEATURE_FLAGS,
                mapOf(ChannelConst.Key.FEATURE_FLAGS to featureFlags)
            )
        }

        override fun setTagId(tagId: String) {
            Log.d("Android", "setTagId not yet implemented")
        }

        override fun notifyCsInAppEnabled(enabled: Boolean) {
            Log.d("Android", "CS InApp feature state changes " + enabled.toString())
            featuresState.setCsInAppState(StateObject(enabled))

            channel.invokeMethod(
                ChannelConst.Method.REQUEST_ENABLE_CS_IN_APP,
                mapOf(ChannelConst.Key.ENABLE_CS_IN_APP to enabled)
            )
        }
    }

    private fun registerNewExternalViewFromActivity(@NonNull activity: Activity) {
        view = activity.findViewById(android.R.id.content) as View


        val flutterView = getFlutterViewIfAvailable() ?: return

        FlutterInterface.excludeExternalView(
            FlutterImageView::class.java,
            FlutterMutatorView::class.java,
        );

        // We register the [FlutterView] as the Flutter root's view even if it is not be the most
        // deepest view on Android side before getting to Flutter part.
        // The reason is that Flutter adds, below the [FlutterView], either a [FlutterSurfaceView]
        // or a [FlutterTextureView] depending on the BackgroundMode of the [FlutterActivity].
        // In order to be sure that we handle both cases (and also potentials future cases of other
        // deeper views), we consider the [FlutterView] as the root of the content created by Flutter.
        flutterInterfaceBridge.registerExternalView(flutterView,
            object : ExternalViewGraphListener {
                override fun takeSnapShot(
                    view: View, path: String, result: ExternalViewGraphResult
                ) {
                    val externalViewGraphResult = result

                    val targetViewPath: String = path + flutterTVPSeparator

                    val pixelRatio: Double =
                        context?.resources?.displayMetrics?.density?.toDouble() ?: 1.0

                    channel.invokeMethod(
                        ChannelConst.Method.REQUEST_FLUTTER_SNAPSHOT,
                        mapOf(
                            ChannelConst.Key.ROOT_PATH to targetViewPath,
                            ChannelConst.Key.PIXEL_RATIO to pixelRatio
                        ),
                        object : Result {
                            override fun success(result: Any?) {
                                externalViewGraphResult.onSnapshotTaken(
                                    view, result as String
                                )
                            }

                            override fun error(
                                errorCode: String, errorMessage: String?, errorDetails: Any?
                            ) {
                                Log.d("Android", "$errorCode, $errorMessage, $errorDetails")
                            }

                            override fun notImplemented() {
                                Log.d("Android", "notImplemented")
                            }
                        })
                }
            })

        xpfInterfaceBridge.registerExternalBridge(externalBridgeInterface)

    }

    private fun getFlutterViewIfAvailable(): View? {
        val flutterView = FlutterViewFinder().find(activity)
        if (flutterView == null) {
            Log.e("Android", "ContentsquarePlugin, no Flutter view found.")
            return null
        }
        return flutterView
    }

    override fun onAttachedToEngine(
        @NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding
    ) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "contentsquare")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext

        executor = MethodCallExecutor()
    }

    override fun onAttachedToActivity(@NonNull activityPluginBinding: ActivityPluginBinding) {
        (activityPluginBinding.lifecycle as HiddenLifecycleReference).lifecycle.addObserver(this)
        activity = activityPluginBinding.activity
    }

    override fun onResume(owner: LifecycleOwner) {
        registerNewExternalViewFromActivity(activity)
    }

    override fun onReattachedToActivityForConfigChanges(@NonNull activityPluginBinding: ActivityPluginBinding) {
        registerNewExternalViewFromActivity(activityPluginBinding.activity)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val agent = MethodCallAgent.of(call.method)
        val data = ExecutionData(call = call, context = context)
        executor.execute(agent = agent, data = data, result = result)
    }

    override fun onDetachedFromEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        context = null
    }

    override fun onDetachedFromActivityForConfigChanges() {
        flutterInterfaceBridge.unRegisterExternalView(view)
        xpfInterfaceBridge.unregisterExternalBridge(externalBridgeInterface)
    }

    override fun onDetachedFromActivity() {
        flutterInterfaceBridge.unRegisterExternalView(view)
        xpfInterfaceBridge.unregisterExternalBridge(externalBridgeInterface)
    }
}
