package com.contentsquare.flutter.infrastructure.bridges

import android.content.Context
import com.contentsquare.android.Contentsquare
import com.contentsquare.android.api.model.CustomVar
import com.contentsquare.android.api.model.Transaction

/**
 * Bridge to `Contentsquare` sdk.
 */
internal interface ContentsquareSDKBridge {
    /**
     * Stops tracking current runtime and wipes user data.
     */
    fun forgetMe()

    /**
     * Returns an URL allowing to visualize the current Session Replay
     * on the CS platform (unless no session is being recorded).
     */
    fun currentSessionReplayLink(): String

    /**
     * The anonymous id used by Contentsquare's SDK to track the current user.
     */
    fun getUserId(): String

    /**
     * Opts the current user's device into being tracked.
     *
     * @param context the android [Context].
     */
    fun optIn(context: Context)

    /**
     * Opts the current user's device out from being tracked.
     *
     * @param context the android [Context].
     */
    fun optOut(context: Context)

    /**
     * Resumes tracking.
     */
    fun resumeTracking()

    /**
     * Stops any further tracking for the current runtime.
     */
    fun stopTracking()

    /**
     * Tracks a dynamic var.
     *
     * @param key the key identifier of the dynamic variable.
     * @param value the integer value of the dynamic variable.
     */
    fun sendDynamicVar(key: String, value: Long)

    /**
     * Tracks a dynamic var.
     *
     * @param key the key identifier of the dynamic variable.
     * @param value the string value of the dynamic variable.
     */
    fun sendDynamicVar(key: String, value: String)

    /**
     * Tracks a screen view with the given name.
     *
     * @param screenName the name of the screenView to track.
     * @param customVars custom variables to track that belong to this screenView
     */
    fun sendScreenName(screenName: String, customVars: Array<CustomVar>)

    /**
     * Tracks the given transaction.
     *
     * @param transaction the [Transaction] to send.
     */
    fun sendTransaction(transaction: Transaction)

    /**
     * Sends a User Identifier to Contentsquare tracking services. This identifier is immediately
     * hashed so no PII can ever be accessed.
     *
     * @param userIdentifier should be max 100 characters long.
     */
    fun sendUserIdentifier(userIdentifier: String)

    companion object Singleton {
        private var instance: ContentsquareSDKBridge? = null

        /**
         * Get the instance of the default implementation of [ContentsquareSDKBridge].
         */
        fun getInstance(): ContentsquareSDKBridge {
            if (instance == null) {
                instance = ContentsquareSDKBridgeImpl()
            }

            return instance as ContentsquareSDKBridge
        }
    }
}


/**
 * Implementation of [ContentsquareSDKBridge] that uses [Contentsquare]
 */
internal class ContentsquareSDKBridgeImpl: ContentsquareSDKBridge {
    override fun forgetMe() {
        Contentsquare.forgetMe()
    }

    override fun currentSessionReplayLink(): String {
        return Contentsquare.currentSessionReplayLink()
    }

    override fun getUserId(): String {
        return Contentsquare.getUserId()
    }

    override fun optIn(context: Context) {
        Contentsquare.optIn(context)
    }

    override fun optOut(context: Context) {
        Contentsquare.optOut(context)
    }

    override fun resumeTracking() {
        Contentsquare.resumeTracking()
    }

    override fun stopTracking() {
        Contentsquare.stopTracking()
    }

    override fun sendDynamicVar(key: String, value: Long) {
        Contentsquare.send(key, value)
    }

    override fun sendDynamicVar(key: String, value: String) {
        Contentsquare.send(key, value)
    }

    override fun sendScreenName(screenName: String, customVars: Array<CustomVar>) {
        Contentsquare.send(screenName, customVars)
    }

    override fun sendTransaction(transaction: Transaction) {
        Contentsquare.send(transaction)
    }

    override fun sendUserIdentifier(userIdentifier: String) {
        Contentsquare.sendUserIdentifier(userIdentifier)
    }

}