package com.contentsquare.flutter.infrastructure.bridges

import android.view.View
import com.contentsquare.android.api.bridge.flutter.ExternalSessionReplayListener
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphListener
import com.contentsquare.android.api.bridge.flutter.FlutterInterface

/**
 * Bridge to FlutterInterface of contentsquare.
 */
internal interface FlutterInterfaceBridge {

    /**
     * Send event.
     *
     * @param json the json of the event.
     */
    fun sendEvent(json: String)

    /**
     * Register a view.
     *
     * @param view the view to register
     * @param listener the listener of some events
     */
    fun registerExternalView(view: View, listener: ExternalViewGraphListener)

    /**
     * Unregister a view.
     *
     * @param view the view to unregister
     */
    fun unRegisterExternalView(view: View)

    /**
     * Register a flutter session replay listener to inform
     * when the session replay is started or stopped.
     *
     * @param listener the listener of session replay state
     */
    fun registerFlutterSessionReplayListener(listener: ExternalSessionReplayListener)

    /**
     * Unregister a flutter session replay listener.
     *
     * @param listener the listener of session replay state
     */
    fun unregisterFlutterSessionReplayListener(listener: ExternalSessionReplayListener)

    /**
     * Send session replay event.
     *
     * @param event the event to send
     * @param pixelRatio the pixel ratio of the device
     */
    fun sendSessionReplayEvent(event: String, pixelRatio: Float)

    /**
     * Send session replay proto data list.
     *
     * @param protoDataList the list of ByteArray representation of FlutterSessionReplay list of events
     *
     */
    // TODO: rename this method in FLUT-1212
    fun sendSessionReplayProtoDataList(protoDataList: List<ByteArray>)

    /**
     * Send crash reports
     *
     * @param reports list of data representation of FlutterThreadReport proto object
     */
    fun sendCrashReports(reports: List<ByteArray>)

    companion object Factory {
        /**
         * Creates a new instance of the default implementation of [FlutterInterfaceBridge].
         */
        fun default(): FlutterInterfaceBridge {
            return FlutterInterfaceBridgeImpl()
        }
    }
}


/**
 * Channel of method in the contentsquare sdk flutter interface.
 */
internal enum class FlutterInterfaceChannel(val value: String) {
    /** channel/method called to send event */
    SEND_EVENT("sendEvent"),

    /** channel/method called to send session replay event */
    SEND_SESSION_REPLAY_EVENT("sendSessionReplayEvent"),

    /** channel/method called to send session replay proto data list */
    SEND_SESSION_REPLAY_PROTO_DATA_LIST("sendSessionReplayProtoDataList"),

    /** channel/method called to register view */
    EXTERNAL_VIEW_REGISTERER("registerExternalView"),

    /** channel/method called to unregister view */
    EXTERNAL_VIEW_UNREGISTERER("unRegisterExternalView"),

    /** channel/method called to register flutter sr listener */
    SESSION_REPLAY_LISTENER_REGISTERER("registerFlutterSrListener"),

    /** channel/method called to unregister flutter sr listener */
    SESSION_REPLAY_LISTENER_UNREGISTERER("unregisterFlutterSrListener"),

    SEND_CRASH_REPORTS("sendCrashReports"),

}


/**
 * Implementation of [FlutterInterfaceBridge].
 *
 * @see FlutterInterfaceBridge
 */
internal class FlutterInterfaceBridgeImpl : FlutterInterfaceBridge {

    override fun sendEvent(json: String) {
        val flutterInterface = FlutterInterface::class.java
        flutterInterface.getDeclaredMethod(
            FlutterInterfaceChannel.SEND_EVENT.value,
            String::class.java
        ).run {
            isAccessible = true
            invoke(null, json)
        }
    }

    override fun registerExternalView(view: View, listener: ExternalViewGraphListener) {
        val flutterInterface = FlutterInterface::class.java
        flutterInterface.getDeclaredMethod(
            FlutterInterfaceChannel.EXTERNAL_VIEW_REGISTERER.value,
            View::class.java,
            ExternalViewGraphListener::class.java
        ).run {
            isAccessible = true
            invoke(null, view, listener)
        }
    }

    override fun unRegisterExternalView(view: View) {
        val flutterInterface = FlutterInterface::class.java
        flutterInterface.getDeclaredMethod(
            FlutterInterfaceChannel.EXTERNAL_VIEW_UNREGISTERER.value, View::class.java
        ).run {
            isAccessible = true
            invoke(null, view)
        }
    }

    override fun registerFlutterSessionReplayListener(listener: ExternalSessionReplayListener) {
            val flutterInterface = FlutterInterface::class.java
            flutterInterface.getDeclaredMethod(
                    FlutterInterfaceChannel.SESSION_REPLAY_LISTENER_REGISTERER.value, ExternalSessionReplayListener::class.java
            ).run {
                isAccessible = true
                invoke(null, listener)
            }
        }

    override fun unregisterFlutterSessionReplayListener(listener: ExternalSessionReplayListener) {
            val flutterInterface = FlutterInterface::class.java
            flutterInterface.getDeclaredMethod(
                    FlutterInterfaceChannel.SESSION_REPLAY_LISTENER_UNREGISTERER.value, ExternalSessionReplayListener::class.java
            ).run {
                isAccessible = true
                invoke(null, listener)
            }
    }

    override fun sendSessionReplayEvent(event: String, pixelRatio: Float) {
        val flutterInterface = FlutterInterface::class.java
        flutterInterface.getDeclaredMethod(
                FlutterInterfaceChannel.SEND_SESSION_REPLAY_EVENT.value, String::class.java, Float::class.java
        ).run {
            isAccessible = true
            invoke(null, event, pixelRatio)
        }
    }

    override fun sendSessionReplayProtoDataList(protoDataList: List<ByteArray>) {
        val flutterInterface = FlutterInterface::class.java
        flutterInterface.getDeclaredMethod(
                FlutterInterfaceChannel.SEND_SESSION_REPLAY_PROTO_DATA_LIST.value, List::class.java
        ).run {
            isAccessible = true
            invoke(null, protoDataList)
        }
    }

    override fun sendCrashReports(reports: List<ByteArray>) {
        val flutterInterface = FlutterInterface::class.java
        flutterInterface.getDeclaredMethod(
            FlutterInterfaceChannel.SEND_CRASH_REPORTS.value, List::class.java
        ).run {
            isAccessible = true
            invoke(null, reports)
        }
    }
}
