package com.contentsquare.flutter.infrastructure.core.method_call_executor.agents

import com.contentsquare.flutter.infrastructure.core.method_call_executor.data.ExecutionData
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.ExecutionResponse
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.FailedExecutionResponse
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.FailedExecutionResponseReason
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.SuccessExecutionResponse
import com.contentsquare.flutter.infrastructure.proxies.ApiErrorData
import com.contentsquare.flutter.infrastructure.proxies.SendApiErrorProxy

/**
 * Agent used to send an [ApiErrorData].
 */
internal class SendApiErrorAgent(private val proxy: SendApiErrorProxy = SendApiErrorProxy.default()) :
    MethodCallAgent(nativeExceptionMessage) {
    companion object {
        /** Message to use in case of error during native call. */
        const val nativeExceptionMessage = "Failed to send ApiError"

        /** Message in case of "url" parameter missing. */
        const val missingUrlMessage = "Missing string for the url parameter"

        /** Message in case of "statusCode" parameter missing. */
        const val missingStatusCodeMessage = "Missing int for the statusCode parameter"

        /** Message in case of "method" parameter missing. */
        const val missingHttpMethodMessage = "Missing string for the method parameter"

        /** Message in case of "query" parameter missing. */
        const val missingQueryMessage = "Missing string for the query parameter"

        /** Message in case of "requestTimeInMs" parameter missing. */
        const val missingRequestTimeMessage = "Missing long for the requestTimeInMs parameter"

        /** Message in case of "responseTimeInMs" parameter missing. */
        const val missingResponseTimeMessage = "Missing long for the responseTimeInMs parameter"
    }

    override fun execute(data: ExecutionData): ExecutionResponse {
        val url = data.call.argument<String>("url")
            ?: return missingArgumentResponse(missingUrlMessage)

        val statusCode = data.call.argument<Int>("statusCode") ?: return missingArgumentResponse(
            missingStatusCodeMessage
        )

        val httpMethod = data.call.argument<String>("method") ?: return missingArgumentResponse(
            missingHttpMethodMessage
        )

        val query = data.call.argument<String>("query") ?: return missingArgumentResponse(
            missingQueryMessage
        )

        val requestTime =
            data.call.argument<Long>("requestTimeInMs") ?: return missingArgumentResponse(
                missingRequestTimeMessage
            )

        val responseTime =
            data.call.argument<Long>("responseTimeInMs") ?: return missingArgumentResponse(
                missingResponseTimeMessage
            )

        val requestBody = data.call.argument<String>("requestBody")

        val responseBody = data.call.argument<String>("responseBody")

        val responseHeaders =
            data.call.argument<Map<String, String>>("responseHeaders")

        val requestHeaders = data.call.argument<Map<String, String>>("requestHeaders")

        val apiError = ApiErrorData(
            url,
            statusCode,
            httpMethod,
            query,
            requestTime,
            responseTime,
            requestBody,
            responseBody,
            responseHeaders,
            requestHeaders
        )
        proxy.send(apiError)

        return SuccessExecutionResponse.empty()
    }

    private fun missingArgumentResponse(message: String): ExecutionResponse {
        return FailedExecutionResponse(
            FailedExecutionResponseReason.MISSING_ARGUMENT,
            message,
            null,
        )
    }
}