package com.contentsquare.flutter.infrastructure.core.platform_views

import android.util.Log
import android.view.View
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.systemchannels.PlatformViewsChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.StandardMethodCodec
import java.lang.ref.WeakReference
import java.util.WeakHashMap
import kotlin.jvm.Throws

/**
 * An interceptor of platform views method calls that is done during the lifecycle of
 * a Flutter application.
 *
 * It should be used to `observe` the different events that is sent from the Dart implementation
 * to the native implementation though the channel `flutter/platform_views`.
 *
 * You should call the method `observe` in the `onAttachedToEngine` of the [FlutterPlugin]
 * and `forget` in the `onDetachedFromEngine`.
 */
internal class PlatformViewsSpy(
    private val supervisor: PlatformViewsSpySupervisor,
    private val viewFromIdProvider: ViewFromIdProvider,
    private val platformViewsMethodCallHandlerProvider: PlatformViewsMethodCallHandlerProvider,
): MethodCallHandler {
    private var platformViewsMethodChannelHandler: WeakReference<MethodCallHandler>? = null
    private var methodChannels = WeakHashMap<FlutterPluginBinding, MethodChannel>()

    /**
     * Launch the observation of the MethodChannel for platform views
     */
    fun observe(pluginBinding: FlutterPluginBinding) {
        try {
            initializePlatformViewsMethodChannelHandler()
            attachEngine(pluginBinding)
        } catch (e: Exception) {
            Log.e("PlatformViewsSpy", e.message ?: e.toString())
        }
    }

    /**
     * Stop the observation of the MethodChannel for platform views.
     *
     * It reverts the method channel to use only the initial MethodCallHandler for platform views.
     */
    fun forget(pluginBinding: FlutterPluginBinding) {
        val handler = platformViewsMethodChannelHandler?.get() ?: return

        methodChannels[pluginBinding]?.setMethodCallHandler(handler)
        platformViewsMethodChannelHandler?.clear()
    }

    @Throws(PlatformViewsInitializationException::class)
    private fun initializePlatformViewsMethodChannelHandler() {
        val handler = platformViewsMethodCallHandlerProvider.getMethodCallHandler()
            ?: throw PlatformViewsInitializationException("Failed to initialize the platform views handler")

        platformViewsMethodChannelHandler = WeakReference(handler)
    }

    private fun attachEngine(pluginBinding: FlutterPluginBinding) {
        val channel = methodChannels[pluginBinding] ?: MethodChannel(
            pluginBinding.binaryMessenger,
            "flutter/platform_views",
            StandardMethodCodec.INSTANCE
        )

        channel.setMethodCallHandler(this)
        methodChannels[pluginBinding] = channel
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        if (call.method == "dispose") {
            handleDisposeMethodCall(call)
        }

        callInitialHandlerAndOverrideResult(
            call = call,
            result = result
        )
    }

    private fun handleDisposeMethodCall(call: MethodCall) {
        if (call.hasArgument("id")) {
            val viewId = call.argument<Int>("id") as Int
            val view = getViewByViewId(viewId)

            supervisor.onReport(
                ViewDisposedSpyReport(
                    view = view,
                    viewId = viewId.toLong()
                )
            )
        }
    }

    private fun getViewByViewId(viewId: Int): View? {
        return viewFromIdProvider.getView(viewId)
    }

    private fun safeReferenceToPlatformViewsMethodChannelHandler(): MethodCallHandler? {
        val handler = platformViewsMethodChannelHandler?.get()

        if (handler != null) {
            return handler
        }

        // As we have a weak reference, the reference could be garbage collected with time
        // Reinitialize it to get the new reference
        initializePlatformViewsMethodChannelHandler()

        return platformViewsMethodChannelHandler?.get()
    }

    private fun callInitialHandlerAndOverrideResult(call: MethodCall, result: MethodChannel.Result) {
        safeReferenceToPlatformViewsMethodChannelHandler()?.onMethodCall(
            call,
            object : MethodChannel.Result {
                override fun success(innerResult: Any?) {
                    if (call.method == "create") {
                        handleCreateMethodCall(call)
                    }

                    result.success(innerResult)
                }

                override fun error(errorCode: String, errorMessage: String?, errorDetails: Any?) {
                    result.error(errorCode, errorMessage, errorDetails)
                }

                override fun notImplemented() {
                    result.notImplemented()
                }

            }
        )
    }

    private fun handleCreateMethodCall(call: MethodCall) {
        if (call.hasArgument("id")) {
            val viewId = call.argument<Int>("id") as Int
            val view = getViewByViewId(viewId)
            if (view != null) {
                supervisor.onReport(
                    ViewCreatedSpyReport(
                        view = view,
                        viewId = viewId.toLong()
                    )
                )
            }
        }
    }
}

/**
 * A provider for the [FlutterEngine] used in the application.
 */
internal interface FlutterEngineProvider {
    fun getFlutterEngine(): FlutterEngine?
}

/**
 * A class used to provide the [View] created through the
 * PlatformView mechanism of Flutter.
 */
internal class ViewFromIdProvider(private val flutterEngineProvider: FlutterEngineProvider) {
    fun getView(viewId: Int): View? {
        return flutterEngineProvider.getFlutterEngine()?.platformViewsController?.getPlatformViewById(viewId)
    }
}

/**
 * A class used to provide the [MethodCallHandler] of Flutter that handles
 * all method calls in the channel `flutter/platform_views` to create the native
 * view.
 */
internal class PlatformViewsMethodCallHandlerProvider(private val flutterEngineProvider: FlutterEngineProvider) {

    /**
     * Get the [MethodCallHandler] used by Flutter to handle calls from the channel
     * `flutter/platform_views`.
     */
    fun getMethodCallHandler(): MethodCallHandler? {
        val controller = flutterEngineProvider.getFlutterEngine()?.platformViewsController ?: return null

        val platformViewsChannel = safelyGetPrivate<PlatformViewsChannel>(
            "platformViewsChannel",
            controller
        ) ?: return null

        return safelyGetPrivate<MethodCallHandler>(
            "parsingHandler",
            platformViewsChannel
        )

    }

    private fun <T> safelyGetPrivate(fieldName: String, obj: Any): T? {
        return try {
            val field = obj.javaClass.getDeclaredField(fieldName)
            field.isAccessible = true

            @Suppress("UNCHECKED_CAST")
            field.get(obj) as T?
        } catch (e: Exception){
            null
        }
    }
}

/**
 * Exception thrown during the initialization of the PlatformViews [MethodCallHandler].
 *
 * If it is thrown, that means the [MethodCallHandler] cannot be retrieved for some reason.
 */
class PlatformViewsInitializationException(message: String?) : RuntimeException(message)

/**
 * Class representing a report of an action made during the observation
 * of the `flutter/platform_views` channel.
 */
internal sealed class SpyReport

/**
 * Implementation of [SpyReport] representing the a report of a [View] created
 * during the handling of the `flutter/platform_views` channel.
 */
internal class ViewCreatedSpyReport(val view: View, val viewId: Long) : SpyReport()

/**
 * Implementation of [SpyReport] representing the a report of a [View] disposed
 * during the handling of the `flutter/platform_views` channel.
 */
internal class ViewDisposedSpyReport(val view: View?, val viewId: Long) : SpyReport()

/**
 * A class that will receive the different [SpyReport].
 */
internal interface PlatformViewsSpySupervisor {
    fun onReport(report: SpyReport)
}