package com.contentsquare.flutter.infrastructure.core.platform_views

import android.webkit.WebView
import com.contentsquare.flutter.infrastructure.bridges.XpfInterfaceBridge
import com.contentsquare.flutter.infrastructure.core.method_call_executor.agents.OnWebViewCreateHandler
import com.contentsquare.flutter.infrastructure.core.method_call_executor.agents.OnWebViewDisposeHandler
import java.lang.ref.WeakReference
import java.util.WeakHashMap

/**
 * It is responsible to handle the registration of a WebView to the
 * Contentsquare SDK through our [XpfInterfaceBridge].
 *
 * It will handle events from the PlatformViews method channel that are reported
 * by the [PlatformViewsSpy] and events send from dart through the plugin MethodChannel
 * so that the WebView will be register and unregister at the expected moment.
 */
internal class WebViewRegistrationManager(
    private val xpfInterfaceBridge: XpfInterfaceBridge = XpfInterfaceBridge.default()
): OnWebViewCreateHandler, PlatformViewsSpySupervisor, OnWebViewDisposeHandler {
    private val registeredWebViews = WeakHashMap<WebView, Long>()
    private val webViewsToRegister = HashMap<Long, WeakReference<WebView>>()
    private val viewIdsToRegister = HashMap<Long, Long>()

    private fun processWebViewRegistration(webView: WebView, webViewId: Long) {
        if (!registeredWebViews.contains(webView)) {
            registeredWebViews[webView] = webViewId
            xpfInterfaceBridge.registerWebView(webView, webViewId)
        }
    }

    private fun processWebViewUnRegistration(webView: WebView) {
        if (registeredWebViews.contains(webView)) {
            xpfInterfaceBridge.unregisterWebView(webView)
            registeredWebViews.remove(webView)
        }
    }

    private fun handleReportedWebViewCreation(webView: WebView, viewId: Long) {
        webViewsToRegister[viewId] = WeakReference(webView)
        viewIdsToRegister[viewId]?.let { webViewId ->
            processWebViewRegistration(webView, webViewId)
        }
    }

    private fun handleReportedWebViewDispose(webView: WebView?, viewId: Long) {
        (webView ?: webViewsToRegister[viewId]?.get())?.let { webView ->
            processWebViewUnRegistration(webView)
        }

        webViewsToRegister.remove(viewId)
        viewIdsToRegister.remove(viewId)
    }

    override fun onWebViewCreate(webViewId: Long, viewId: Long) {
        webViewsToRegister[viewId]?.get()?.let { webViewToRegister ->
            processWebViewRegistration(webViewToRegister, webViewId)
        } ?: run {
            viewIdsToRegister[viewId] = webViewId
        }
    }

    override fun onReport(report: SpyReport) {
        when(report) {
            is ViewDisposedSpyReport -> {
                handleReportedWebViewDispose(report.view as? WebView, report.viewId)
            }

            is ViewCreatedSpyReport -> {
                if (report.view is WebView) {
                    handleReportedWebViewCreation(report.view, report.viewId)
                }
            }
        }
    }

    override fun onWebViewDispose(webViewId: Long, viewId: Long) {
        webViewsToRegister[viewId]?.get()?.let { webView ->
            processWebViewUnRegistration(webView)
        }

        webViewsToRegister.remove(viewId)
        viewIdsToRegister.remove(viewId)
    }
}