package com.contentsquare.flutter.infrastructure.proxies

import com.contentsquare.flutter.infrastructure.bridges.ErrorAnalysisBridge
import com.contentsquare.android.error.analysis.ErrorAnalysis

/**
 * Allows to send [ApiErrorData] to the native SDK.
 */
internal interface SendApiErrorProxy {
    /**
     * Send arguments data (representing an [ApiErrorData]) to the native SDK.
     */
    fun send(data: ApiErrorData)

    companion object Factory {
        /**
         * Creates a new instance of the default implementation of [SendApiErrorProxy].
         */
        fun default(errorAnalysis: ErrorAnalysisBridge = ErrorAnalysisBridge.getInstance()): SendApiErrorProxy {
            return SendApiErrorProxyImpl(errorAnalysis = errorAnalysis)
        }
    }
}


/**
 * An implementation of [SendApiErrorProxy] that sends the [ApiErrorData] data to the native SDK.
 *
 * @see SendApiErrorProxy
 */
internal class SendApiErrorProxyImpl(private val errorAnalysis: ErrorAnalysisBridge) :
    SendApiErrorProxy {
    override fun send(data: ApiErrorData) {
        val networkMetric = ErrorAnalysis.getInstance()
            .newNetworkMetric(data.url, data.httpMethod).apply {
                setStatusCode(data.statusCode)
                setRequestTime(data.requestTime)
                setResponseTime(data.responseTime)

                data.requestBody?.let { setRequestBody(it.toByteArray()) }
                data.responseBody?.let { setResponseBody(it.toByteArray()) }
                data.requestHeaders?.let { setRequestHeaders(it) }
                data.responseHeaders?.let { setResponseHeaders(it) }
            }
        errorAnalysis.sendApiError(networkMetric)
    }

}


/**
 * Class that holds the data of an ApiError
 * */
internal data class ApiErrorData(
    /** Represents the request url of the request that failed.*/
    val url: String,

    /** The status code of the http response.*/
    val statusCode: Int,

    /** An http method. Can be GET, POST, PUT, DELETE, PATCH, HEAD, OPTIONS, TRACE, CONNECT.*/
    val httpMethod: String,

    /** String representation of the query parameters.*/
    val query: String,

    /** Time in milliseconds since epoch when the error request was sent.*/
    val requestTime: Long,

    /** Time in milliseconds since epoch when the error response was received.*/
    val responseTime: Long,

    /** String representation of the request body.*/
    val requestBody: String?,

    /** String representation of the response body.*/
    val responseBody: String?,

    /** Represents the request headers of the request that failed.*/
    val requestHeaders: Map<String, String>?,

    /** Represents the response headers of the response that failed.*/
    val responseHeaders: Map<String, String>?,
)