package com.contentsquare.flutter.infrastructure.proxies


import com.contentsquare.android.api.model.CustomVar
import com.contentsquare.flutter.infrastructure.bridges.ContentsquareSDKBridge

/**
 * Allows to send a screen name to Contentsquare API.
 */
internal interface SendScreenNameProxy {
    /**
     * Send the name of the screen to contentsquare API.
     *
     * @param screenName the name of the screen to send.
     * @param customVars custom variables specific to this screen to send.
     */
    fun send(screenName: String, customVariables: Array<CustomVariable>)

    companion object Factory {
        /**
         * Creates a new instance of the default implementation of [SendScreenNameProxy].
         */
        fun default(contentsquareSDK: ContentsquareSDKBridge = ContentsquareSDKBridge.getInstance()): SendScreenNameProxy {
            return SendScreenNameProxyImpl(contentsquareSDK = contentsquareSDK)
        }
    }
}

/**
 * Custom variable sent with the screen
 * */
internal data class CustomVariable(
        /** A number identifying the variable.*/
        val index: Int,
        /** A description of what you want to track.*/
        val name: String,
        /** The string value taken by this occurrence of the var.*/
        val value: String,
)

/**
 * An implementation of [SendScreenNameProxy] that sends the screen name to the Content Square API.
 *
 * @see SendScreenNameProxy
 */
internal class SendScreenNameProxyImpl(private val contentsquareSDK: ContentsquareSDKBridge) : SendScreenNameProxy {
    override fun send(screenName: String, customVariables: Array<CustomVariable>) {
        contentsquareSDK.sendScreenName(screenName, mapCustomVariableArrayToCustomVarArray(customVariables))
    }

    private fun mapCustomVariableArrayToCustomVarArray(customVariables: Array<CustomVariable>) = customVariables.map {
        CustomVar(
                index = it.index,
                name = it.name,
                value = it.value,
        )
    }.toTypedArray()

}
