import ContentsquareModule
import WebKit

/// Bridge to `Contentsquare` sdk.
protocol ContentsquareSDKBridge {
    /// Tracks a screen view with the given name.
    ///
    /// - Parameter name: The name of the screenView to track.
    /// - Parameter cvars: Array of custom variables giving additional information about the named screen
    func sendScreen(named name: String, withVars cvars: [CustomVar]) throws -> Void
    
    /// Tracks a dynamic var.
    ///
    /// - Parameter dynamicVar: contains a pair of arbitrary key/value to be sent using this method.
    func sendDynamicVar(dynamicVar: DynamicVar) throws -> Void
    
    /// Tracks the given transaction.
    ///
    /// - Parameter transaction: Create a `CustomerTransaction` with a value, a currency and an optional id
    ///     and send it using this method.
    func sendTransaction(transaction: CustomerTransaction) throws -> Void
    
    /// Opts the current user's device out from being tracked.
    func optOut() throws -> Void
    
    /// Opts the current user's device into being tracked.
    func optIn() throws -> Void
    
    /// Stops tracking current runtime and wipes user data.
    func forgetMe() throws -> Void
    
    /// The anonymous id used by Contentsquare's SDK to track the current user.
    func getUserID() throws -> String?
    
    /// Stops any further tracking for the current runtime.
    func stopTracking() throws -> Void
    
    /// Resumes tracking.
    func resumeTracking() throws -> Void
    
    /// Returns an URL allowing to visualize the current Session Replay
    /// on the CS platform (unless no session is being recorded).
    func getCurrentSessionReplayLink() throws -> URL?
    
    /// Send touch event.
    func sendTouchEvent(body: String) throws -> Void
    
    /// Send the session replay event.
    func sendSessionReplayEvent(body: [String: Any]) throws -> Void
    
    /// Send the session replay event as Data list.
    func sendSessionReplayEvent(events: [Data]) throws -> Void
    
    /// Sends a User Identifier to Contentsquare tracking services. This
    /// identifier is immediately hashed so no PII can ever be accessed.
    ///
    /// - Parameter userIdentifier: the user identifier should be max 100
    /// characters long.
    func sendUserIdentifier(named userIdentifier: String) throws -> Void

    /// Send Flutter crash reports 
    func sendFlutterCrashReports(reports: [Data]) throws -> Void
    
    /// Register a [WKWebView]
    func registerWebView(webView: WKWebView) throws -> Void
    
    /// Unregister a [WKWebView]
    func unregisterWebView(webView: WKWebView) throws -> Void
}

/// Factory creator class for [ContentsquareSDKBridge].
struct ContentsquareSDKBridgeFactory {
    
    private static var _instance: ContentsquareSDKBridge? = nil

    /// Creates a new instance of the default implementation of [ContentsquareSDKBridge].
    static func getInstance() -> ContentsquareSDKBridge {

        if (_instance == nil) {
            _instance = ContentsquareSDKBridgeImpl()
        }
        
        return _instance!
    }
}


/// An implementation of [ContentsquareSDKBridge]
class ContentsquareSDKBridgeImpl: ContentsquareSDKBridge {
    
    
    func sendSessionReplayEvent(events: [Data]) throws {
        Contentsquare.perform(NSSelectorFromString("_sendSessionReplayEventsWithProtoDataList:"),
        with: events)
    }
    
    func sendSessionReplayEvent(body: [String : Any]) throws {
        Contentsquare.perform(NSSelectorFromString("_sendSessionReplayEventWithParameters:"), with: body)
    }
    
    func sendTouchEvent(body: String) throws -> Void {
        Contentsquare.perform(NSSelectorFromString("_sendEventWithJson:"), with: body)
    }
    
    func getCurrentSessionReplayLink() throws -> URL? {
        return Contentsquare.currentSessionReplayLink
    }
    
    func resumeTracking() throws {
        Contentsquare.resumeTracking()
    }
    
    func stopTracking() throws {
        Contentsquare.stopTracking()
    }
    
    func getUserID() throws -> String? {
        return Contentsquare.userID
    }
    
    func forgetMe() throws {
        Contentsquare.forgetMe()
    }
    
    func sendScreen(named name: String, withVars cvars: [ContentsquareModule.CustomVar]) throws {
        Contentsquare.send(screenViewWithName: name, cvars: cvars)
    }
    
    func sendDynamicVar(dynamicVar: ContentsquareModule.DynamicVar) throws {
        Contentsquare.send(dynamicVar: dynamicVar)
    }
    
    func sendTransaction(transaction: ContentsquareModule.CustomerTransaction) throws {
        Contentsquare.send(transaction: transaction)
    }
    
    func optOut() throws {
        Contentsquare.optOut()
    }
    
    func optIn() throws {
        Contentsquare.optIn()
    }
    
    func sendUserIdentifier(named userIdentifier: String) throws -> Void {
        Contentsquare.sendUserIdentifier(userIdentifier)
    }

    func sendFlutterCrashReports(reports: [Data]) throws {
        Contentsquare.perform(NSSelectorFromString("_sendFlutterCrashReportsWithProtoDataList:"), with: reports)
    }
    
    func registerWebView(webView: WKWebView) throws {
        Contentsquare.register(webView: webView)
    }
    
    func unregisterWebView(webView: WKWebView) throws {
        Contentsquare.unregister(webView: webView)
    }
    
}
