/// `MethodCallAgent` used to execute send api error.
class SendApiErrorAgent: MethodCallAgent {
    
    internal init(proxy: SendApiErrorProxy = SendApiErrorProxyFactory.makeDefault()) {
        self._proxy = proxy
    }
    
    /// Message to use in case of error during native call.
    static let exceptionMessage = "Exception occurred while sending api error"
    
    /// Get the message in case of missing value for a key.
    static func missingValueForKeyMessage(key: String) -> String {
        return "Missing value for key '\(key)'"
    }
    
    /// Get the message for invalid URL.
    static func invalidURLMessage(urlString: String) -> String {
        return "Invalid URL '\(urlString)'"
    }
    
    /// Get the message for invalid data.
    static func invalidDataMessage(key: String) -> String {
        return "Invalid Data '\(key)'"
    }
    
    /// Get the message for unexpected error.
    static func unexpectedErrorMessage(error: Error) -> String {
        return "Unexpected error: '\(error)'"
    }
    
    private let _proxy: SendApiErrorProxy
    
    var nativeExceptionMessage: String? {
        return SendApiErrorAgent.exceptionMessage
    }
    
    func execute(data: ExecutionData) throws -> ExecutionResponse {
        let arguments: [String : Any] = data.call.arguments as? [String : Any] ?? [:]
        
        do {
            try _proxy.sendApiError(ofValue: arguments)
            return ExecutionResponse.succeed(result: nil)
        } catch {
            let (code, message): (FailedExecutionResponseReason, String)
                switch error {
                    case HttpMetricConverterError.missingValue(let key):
                        code = FailedExecutionResponseReason.missingArgument
                        message = SendApiErrorAgent.missingValueForKeyMessage(key: key)
                    case HttpMetricConverterError.invalidURL(let urlString):
                        code = FailedExecutionResponseReason.invalidURL
                        message = SendApiErrorAgent.invalidURLMessage(urlString: urlString)
                    case HttpMetricConverterError.invalidData(forKey: let key):
                        code = FailedExecutionResponseReason.invalidData
                        message = SendApiErrorAgent.invalidDataMessage(key: key)
                    default:
                        code = FailedExecutionResponseReason.unexpectedError
                        message = SendApiErrorAgent.unexpectedErrorMessage(error: error)
                }
            throw FailedExecutionError.init(reason: code, message: message, cause: nil)
        }
        
    }
    
}
