/// `MethodCallAgent` used to handle WebView create event
class WebViewCreateAgent: MethodCallAgent {
    
    internal init(handler: OnWebViewCreateHandler) {
        self._handler = handler
    }
    
    /// Message to use in case of error during native call.
    static let exceptionMessage = "Error while handling the WebView creation event."
    
    /// Message in case of "user_script" parameter missing
    static let missingUserScriptParameter =  "Missing user_script parameter."
    
    /// Message in case of "channel_name" parameter missing
    static let missingChannelNameParameter =  "Missing channel_name parameter."
    
    private let _handler: OnWebViewCreateHandler
    
    var nativeExceptionMessage: String? {
        return WebViewCreateAgent.exceptionMessage
    }
    
    func execute(data: ExecutionData) throws -> ExecutionResponse {
        let arguments: [String : Any] = data.call.arguments as? [String : Any] ?? [:]
        
        guard let userScript = arguments["user_script"] as? String else {
            throw FailedExecutionError(
                reason: FailedExecutionResponseReason.missingArgument,
                message: WebViewCreateAgent.missingUserScriptParameter,
                cause: nil
            )
        }
        
        guard let channelName = arguments["channel_name"] as? String else {
            throw FailedExecutionError(
                reason: FailedExecutionResponseReason.missingArgument,
                message: WebViewCreateAgent.missingChannelNameParameter,
                cause: nil
            )
        }
        
        self._handler.onWebViewCreate(userScript: userScript, channelName: channelName)
        
        return ExecutionResponse.succeed(result: nil)
    }
    
}


/// The class that will handle the event of WebView creation.
protocol OnWebViewCreateHandler: AnyObject {
    
    /// Inform of the WebView creation event
    ///
    /// - Parameter userScript: The [WKUserScript] source that is injected in the WebView
    /// - Parameter channelName: The name of channel injected in the WebView as user script
    func onWebViewCreate(userScript: String, channelName: String)
}
