import WebKit

/// It is responsible to handle the registration of a WebView to the
/// Contentsquare SDK through the [ContentsquareSDKBridge].
class WebViewRegistrationManager: WebViewFoundHandler, OnWebViewCreateHandler, OnWebViewDisposeHandler {
    
    private let _contentsquareSDKBridge: ContentsquareSDKBridge
    private weak var _flutterRootViewProvider: FlutterRootViewProvider?
    
    private var _registeredWebViews: [String: WeakReference<WKWebView>]
    private var _webViewScanners: [String: WebViewScanner]
    
    init(contentsquareSDKBridge: ContentsquareSDKBridge, flutterRootViewProvider: FlutterRootViewProvider, registeredWebViews: [String : WeakReference<WKWebView>] = [:]) {
        self._contentsquareSDKBridge = contentsquareSDKBridge
        self._flutterRootViewProvider = flutterRootViewProvider
        self._registeredWebViews = registeredWebViews
        self._webViewScanners = [:]
    }
    
    func onWebViewCreate(userScript: String, channelName: String) {
        guard let targetView = _flutterRootViewProvider?.getView() else {
            return
        }
        let scanner = _webViewScanners[userScript] ?? WebViewScanner(queryScript: userScript, channelName: channelName, targetView: targetView, handler: self)
        scanner.start()
        _webViewScanners[userScript] = scanner
    }
    
    func handleFoundWebView(webView: WKWebView, queryScript: String) {
        if _registeredWebViews.keys.contains(queryScript) {
            return
        }
        
        do {
            _webViewScanners.removeValue(forKey: queryScript)
            try _contentsquareSDKBridge.registerWebView(webView: webView)
            _registeredWebViews[queryScript] = WeakReference(value: webView)
        } catch {
            print("CSLIB: \(error.localizedDescription)")
        }
    }
    
    func getWebViewScanner(for userScript: String) -> WebViewScanner? {
        return _webViewScanners[userScript]
    }
    
    func onWebViewDispose(userScript: String, channelName: String) {
        getWebViewScanner(for: userScript)?.stop()
        
        if let webView = _registeredWebViews[userScript]?.value as? WKWebView {
            
            do {
                try _contentsquareSDKBridge.unregisterWebView(webView: webView)
                _registeredWebViews.removeValue(forKey: userScript)
            } catch {
                print("CSLIB: \(error.localizedDescription)")
            }
        }
    }
    
}
