import WebKit

class WebViewScanner {
    let queryScript: String
    private let _webViewFlutterPackageChannelName: String
    private let _subWebViewsFinder: SubWebViewsFinder
    private let _webViewUserScriptGetter: WebViewUserScriptGetter
    private weak var _targetView: UIView?
    private weak var _handler: WebViewFoundHandler?
    
    private var _isScanning: Bool = false
    
    private let _intervalScan: TimeInterval
    private let _timeoutScan: TimeInterval
    
    private var _scanInterval: Timer?
    private var _scanTimeout: Timer?
    
    var isWorking: Bool {
        get {
            return _scanInterval?.isValid == true
        }
    }
    
    init(queryScript: String, channelName: String, targetView: UIView, handler: WebViewFoundHandler, finder: SubWebViewsFinder = _DefaultSubWebViewsFinder(), intervalScan: TimeInterval = 0.25, timeoutScan: TimeInterval = 5.0, webViewUserScriptGetter: WebViewUserScriptGetter = _DefaultWebViewUserScriptGetter()) {
        self.queryScript = queryScript
        self._targetView = targetView
        self._handler = handler
        // Script/Channel name injected through `addJavaScriptChannel` from the
        // package `webview_flutter` are injected in the userScripts
        // but with the prefix "window."
        self._webViewFlutterPackageChannelName = "window." + channelName
        self._intervalScan = intervalScan
        self._timeoutScan = timeoutScan
        self._subWebViewsFinder = finder
        self._webViewUserScriptGetter = webViewUserScriptGetter
    }
    
    deinit {
        stop()
    }
    
    func start() {
        // Ensure that no existing timers/scans are running
        if isWorking {
            stop()
        }
        
        // Start the scanning timer
        _scanInterval = Timer.scheduledTimer(withTimeInterval: _intervalScan, repeats: true) { [weak self] _ in
            self?._perform()
        }
        
        // Start the timeout timer of [_timeoutScan] seconds to avoid infinite running
        _scanTimeout = Timer.scheduledTimer(withTimeInterval: _timeoutScan, repeats: false) { [weak self] _ in
            self?.stop()
        }
    }
    
    func stop() {
        _scanInterval?.invalidate()
        _scanInterval = nil
        _scanTimeout?.invalidate()
        _scanTimeout = nil
        _isScanning = false
    }
    
    private func _perform() {
        if _isScanning {
            return
        }
        
        
        guard let viewToScan = _targetView else {
            return
        }
        
        _isScanning = true
        let allWebViews = _subWebViewsFinder.getSubWebViews(of: viewToScan)
        for webView in allWebViews {
            let hasQueryScript = _webViewUserScriptGetter.getUserScripts(of: webView).contains { script in
                let source = script.source
                return source.starts(with: queryScript) || source.starts(with: _webViewFlutterPackageChannelName)
            }
            
            if hasQueryScript {
                _handler?.handleFoundWebView(webView: webView, queryScript: queryScript)
                stop()
                return
            }
        }
        _isScanning = false
    }
}

/// The handler of the WebView found during the scan of the subviews.
protocol WebViewFoundHandler: AnyObject {
    
    /// Handle the WebView that is found during the scan.
    ///
    /// - Parameter webView: The WebView found during the scan
    /// - Parameter queryScript: The query that was searched inside the userScripts of the WebView
    func handleFoundWebView(webView: WKWebView, queryScript: String)
}

/// A class that help to find the WebViews that are subviews of a view.
protocol SubWebViewsFinder: AnyObject {
    
    /// Get the subviews of the view that are type of [WKWebView]
    ///
    /// - Parameter view: The view to get the subviews of type WKWebView
    func getSubWebViews(of view: UIView) -> [WKWebView]
}

class _DefaultSubWebViewsFinder: SubWebViewsFinder {
    func getSubWebViews(of view: UIView) -> [WKWebView] {
        return view.allSubviews { view in
            view is WKWebView
        } as? [WKWebView] ?? []
    }
}

/// A class that get the user scripts of a [WKWebView]
protocol WebViewUserScriptGetter: AnyObject {
    
    /// Get the user scripts of a [WKWebView]
    ///
    /// - Parameter webView: The webView in which we need the user scripts
    func getUserScripts(of webView: WKWebView) -> [WKUserScript]
}

class _DefaultWebViewUserScriptGetter: WebViewUserScriptGetter {
    func getUserScripts(of webView: WKWebView) -> [WKUserScript] {
        return webView.configuration.userContentController.userScripts
    }
    
}
