import ContentsquareModule

/// Allows to send a screen name to Contentsquare API.
protocol SendScreenNameProxy {
    /// Send the name of the screen to contentsquare API.
    ///
    /// - Parameter screenName: the name of the screen to send
    func send(screenName: String, vars: [CustomVariable]) throws -> Void
}

/// Custom variable send with the screen.
struct CustomVariable {
    let index: NSNumber
    let name: String
    let value: String
    
    /// Creates a new instance of [CustomVariable]
    ///
    /// - Parameter index: a number identifying the variable.
    /// - Parameter name: A description of what you want to track.
    /// - Parameter value: The string value taken by this occurrence of the var.
    init(index: NSNumber, name: String, value: String) {
        self.index = index
        self.name = name
        self.value = value
    }
}

/// Factory creator class for [SendScreenNameProxy].
struct SendScreenNameProxyFactory {

    /// Creates a new instance of the default implementation of [SendScreenNameProxy].
    ///
    /// - Parameter contensquareSDK: The `ContentsquareSDKBridge` used to access contentsquare API.
    static func makeDefault(contensquareSDK: ContentsquareSDKBridge = ContentsquareSDKBridgeFactory.getInstance()) -> SendScreenNameProxy {
        return SendScreenNameProxyImpl(contensquareSDK: contensquareSDK)
    }
}

/// An implementation of [SendScreenNameProxy]
class SendScreenNameProxyImpl: SendScreenNameProxy {
    
    private let _contensquareSDK: ContentsquareSDKBridge
    
    /// Creates a new instance of `SendTransactionProxyImpl`
    ///
    /// - Parameter contensquareSDK: The `ContentsquareSDKBridge` used to access contentsquare API.
    internal init(contensquareSDK: ContentsquareSDKBridge) {
        self._contensquareSDK = contensquareSDK
    }
    
    func send(screenName: String, vars: [CustomVariable] = []) throws {
        try _contensquareSDK.sendScreen(named: screenName, withVars: _mapCustomVariableListToCustomVarList(variables: vars))
    }
    
    private func _mapCustomVariableListToCustomVarList(variables: [CustomVariable]) -> [CustomVar] {
        return variables.map { CustomVar(index: UInt32($0.index.intValue), name: $0.name, value: $0.value) }
    }
}
